#!/usr/bin/env python
from CeboMsrApiPython import LibraryInterface, DeviceType
from math import sin

# CeboMsr - Python example - Frame I/O.
#
# Multiple inputs are read multiple times in a single transaction,
# values are printed to the console.
# AO-1 and DP-2 are set multiple times in a single transaction.
#
# Cebo-Stick example does input transfer only.
#
# @author Thomas Hoppe

def frameIoTestLC(device):
    """
    Do frame IO (read/write multiple I/O in single calls).
    device: Device to use for test.
    """
    # Prepare and fill the list of inputs to read.
    inputs = [
        device.getSingleEndedInputs()[0],
        device.getSingleEndedInputs()[1],
        device.getDifferentialInputs()[1],
        device.getDigitalPorts()[0],
        device.getDigitalPorts()[1],
        device.getCounters()[0]
    ]

    # Setup device with this selection.
    device.setupInputFrame(inputs)

    # Read the values multiple times and write them to the console.
    for _ in range(100):
        # Read all inputs into the instance of InputFrame.
        inFrame = device.readFrame()

        # Write results to the console.
        print(
            ("DigitalPort #0: %d" % inFrame.getDigitalPort(0)) + ", " +
            ("DigitalPort #1: %d" % inFrame.getDigitalPort(1)) + ", " +
            ("SingleEnded #0: %.2f" % inFrame.getSingleEnded(0)) + " V, " +
            ("SingleEnded #1: %.2f" % inFrame.getSingleEnded(1)) + " V, " +
            ("Differential #1: %.2f" % inFrame.getDifferential(1)) + " V, " +
            ("Counter #0: %d" % inFrame.getCounter(0)))

    # Write to analog out #1 and digital out #2 in one call.
    # Prepare and set output selection.
    outputs = [
        device.getDigitalPorts()[2],
        device.getAnalogOutputs()[1]
    ]

    # Prepare device ...
    device.setupOutputFrame(outputs)

    # Prepare digital port #2 as output.
    device.getDigitalPorts()[2].setOutputEnableMask(0xf)

    # Create instance of OutputFrame. There's no direct construction, as this
    # instance may vary between different device types.
    outFrame = device.createOutputFrame()

    # Write it to hardware, modify contents in every looping.
    for i in range(100):
        outFrame.setAnalogOutput(1, float(3 * sin(float(i) / 100)))
        outFrame.setDigitalPort(2, i)

        device.writeFrame(outFrame)

    # Reset to power up defaults.
    device.resetDevice()

def frameIoTestStick(device):
    """
    Do frame IO (read/write multiple I/O in single calls).
    device: Device to use for test.
    """
    # Prepare and fill the list of inputs to read.
    inputs = [
        device.getSingleEndedInputs()[0],
        device.getSingleEndedInputs()[1],
        device.getDigitalPorts()[0],
        device.getCounters()[0]
    ]

    # Setup device with this selection.
    device.setupInputFrame(inputs)

    # Read the values multiple times and write them to the console.
    for _ in range(100):
        # Read all inputs into the instance of InputFrame.
        inFrame = device.readFrame()

        # Write results to the console.
        print(
            ("DigitalPort #0: %d" % inFrame.getDigitalPort(0)) + ", " +
            ("SingleEnded #0: %.2f" % inFrame.getSingleEnded(0)) + " V, " +
            ("SingleEnded #1: %.2f" % inFrame.getSingleEnded(1)) + " V, " +
            ("Counter #0: %d" % inFrame.getCounter(0)))

    # Reset to power up defaults.
    device.resetDevice()

def main():
    """
    The examples main method.
    """
    try:
        # Search for devices ...
        devices = LibraryInterface.enumerate(DeviceType.All)

        # If at least one has been found, use the first one ...
        if (len(devices) > 0):
            device = devices[0]

            # Open device, nothing can be done without doing this.
            device.open()

            # *** DIGITAL PORTS AND ANALOG OUTPUTS ARE MODIFIED DURING THIS ***
            # *** SECTION, SO DO NOT CONNECT ANY PERIPHERALS WITHOUT STUDYING ***
            # *** THE ACTIONS DONE HERE, OTHERWISE YOUR PERIPHERALS MAY BE DAMAGED !!! ***
            if (device.getDeviceType() == DeviceType.CeboLC):
                frameIoTestLC(device)
            elif (device.getDeviceType() == DeviceType.CeboStick):
                frameIoTestStick(device)
            else:
                raise SystemError("No supported device detected.")

            # Finalize device usage, this free's up the device, so it can be used
            # again, including other applications.
            device.close()
    except Exception as e:
        print(e)

if (__name__ == "__main__"):
    main()
