package com.cesys.cebo.cebomsr;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Offers functionality of the different kind of analog inputs.
 * Direct construction is not possible, instances can be retrieved calling
 * getSingleEndedInputs() or getDifferentialInputs() of the respective instance
 * of class {@link Device}.
 *
 * @author Thomas Hoppe
 */
public final class AnalogInput implements Input {
	enum Style {
		SingleEnded,
		Differential
	}

	private final List<Range> _supportedRanges = new ArrayList<Range>();
	private final Device _device;
	private final int _interfaceId;
	private final Style _style;
	private final int _index;
	private final int _minInterChannelDelay;
	private final String _name;

	int getInterfaceId() {
		return _interfaceId;
	}

	Style getStyle() {
		return _style;
	}

	int getIndex() {
		return _index;
	}

	AnalogInput(Device device, int interfaceId, Style style, int index) throws IOException {
		_device = device;
		_interfaceId = interfaceId;
		_style = style;
		_index = index;

		int[] rangeId = new int[1];
		for (int i = 0;; ++i) {
			SystemConnector.checkError(SystemConnector.CeboMsrEnumerateRanges(_device.getHandle(), getInterfaceId(), i, rangeId));
			if (rangeId[0] == SystemConnector.CeboMsrRangeInvalid)
				break;

			float[] minValue = new float[1], maxValue = new float[1];
			int[] interChannelDelay = new int[1];
			SystemConnector.checkError(SystemConnector.CeboMsrGetRangeMinMax(device.getHandle(), getInterfaceId(), rangeId[0], minValue, maxValue));
			SystemConnector.checkError(SystemConnector.CeboMsrGetDefaultAnalogInputInterChannelDelay(_device.getHandle(), getInterfaceId(), rangeId[0], interChannelDelay));
			_supportedRanges.add(new Range(rangeId[0], minValue[0], maxValue[0], interChannelDelay[0]));
		}
		int[] micd = new int[1];
		SystemConnector.checkError(SystemConnector.CeboMsrGetMinAnalogInputInterChannelDelay(_device.getHandle(), getInterfaceId(), micd));
		_minInterChannelDelay = micd[0];

		byte[] tmp = new byte[SystemConnector.MaxIdentLength];
		SystemConnector.checkError(SystemConnector.CeboMsrGetPeripheralName(device.getHandle(), interfaceId, tmp));
		_name = Utils.convertByteArrayToString(tmp);
	}

	/**
	 * Return list of the supported ranges.
	 */
	public List<Range> getSupportedRanges() {
		return Collections.unmodifiableList(_supportedRanges);
	}

	/**
	 * Request default interchannel delay at the given range setting.
	 * @param range Range for which this value is requested.
	 */
	public int getDefaultInterChannelDelay(Range range) throws IOException {
		return range.getDefaultInterChannelDelay();
	}

	/**
	 * Return minimum interchannel delay.
	 */
	public int getMinInterChannelDelay() {
		return _minInterChannelDelay;
	}

	/**
	 * Set new range, interchannel delay is set to the default for this range.
	 * @param range Range level to set.
	 */
	public void setParameters(Range range) throws IOException {
		int r[] = new int[1];
		SystemConnector.checkError(SystemConnector.CeboMsrSetAnalogInputParameters(_device.getHandle(), getInterfaceId(), range.getInterfaceId(), -1, r));
	}

	/**
	 * Set range and user specified interchannel delay.
	 * @param range Range level to set.
	 * @param interChannelDelay Interchannel delay in microseconds.
	 * @return Corrected interchannel delay in microseconds.
	 */
	public int setParameters(Range range, int interChannelDelay) throws IOException {
		int r[] = new int[1];
		SystemConnector.checkError(SystemConnector.CeboMsrSetAnalogInputParameters(_device.getHandle(), getInterfaceId(), range.getInterfaceId(), interChannelDelay, r));
		return r[0];
	}

	/**
	 * Read active range level.
	 */
	public Range getRange() throws IOException {
		int[] g = new int[1], st = new int[1];
		SystemConnector.checkError(SystemConnector.CeboMsrGetAnalogInputParameters(_device.getHandle(), getInterfaceId(), g, st));
		for (Range range: _supportedRanges)
			if (range.getInterfaceId() == g[0])
				return range;
		throw new IllegalStateException("Invalid range, internal error.");
	}

	/**
	 * Read active interchannel delay.
	 */
	public int getInterChannelDelay() throws IOException {
		int[] g = new int[1], st = new int[1];
		SystemConnector.checkError(SystemConnector.CeboMsrGetAnalogInputParameters(_device.getHandle(), getInterfaceId(), g, st));
		return st[0];
	}

	/**
	 * Read current voltage value.
	 */
	public float read() throws IOException {
		float[] r = new float[1];
		SystemConnector.checkError(SystemConnector.CeboMsrReadAnalogInput(_device.getHandle(), getInterfaceId(), r));
		return r[0];
	}

	/**
	 * Returns name of the component.
	 */
	public String getName() {
		return _name;
	}

	@Override
	public String toString() {
		return Style.Differential == _style ? ("Differential" + _index) : ("SingleEnded" + _index);
	}
}
