package com.cesys.cebo.cebomsr.examples;

import java.io.IOException;
import java.util.List;

import com.cesys.cebo.cebomsr.Device;
import com.cesys.cebo.cebomsr.DeviceType;
import com.cesys.cebo.cebomsr.Input;
import com.cesys.cebo.cebomsr.InputFrame;
import com.cesys.cebo.cebomsr.LibraryInterface;

/**
 * CeboMsr - Java example - Streaming Input.
 *
 * Multiple inputs are read multiple times at 300 Hz
 * and printed to the console.
 *
 * @author Thomas Hoppe
 */
public class StreamingIExample {
	/**
	 * Read blocks of data from device.
	 * @param device Device to use for test.
	 * @throws IOException
	 */
	private void streamingInputTestLC(Device device) throws IOException {
		// Construct selection that contains inputs to read from.
		Input[] inputs = new Input[] {
			device.getSingleEndedInputs().get(0),
			device.getSingleEndedInputs().get(1),
			device.getDifferentialInputs().get(1),
			device.getDigitalPorts().get(0),
			device.getDigitalPorts().get(1)
		};

		// Prepare device with this collection ...
		device.setupInputFrame(inputs);

		// Start sampling ...
		device.startBufferedDataAcquisition(300, 20 * 25, false);

		// Read 20 x 25 frames using blocked read,
		// this function returns after *all* (25) requested frames are collected.
		for (int i = 0; i < 20; ++i) {
			// Read 25 frames ...
			List<InputFrame> frames = device.readBlocking(25);

			// Write out the 1st one.
			InputFrame inFrame = frames.get(0);
			System.out.println(
				"DigitalPort #0: " + inFrame.getDigitalPort(0) + ", " +
				"DigitalPort #1: " + inFrame.getDigitalPort(1) + ", " +
				"SingleEnded #0: " + inFrame.getSingleEnded(0) + " V, " +
				"SingleEnded #1: " + inFrame.getSingleEnded(1) + " V, " +
				"Differential #1: " + inFrame.getDifferential(1) + " V");
		}

		// Stop the DAQ.
		device.stopDataAcquisition();
	}

	/**
	 * Read blocks of data from device.
	 * @param device Device to use for test.
	 * @throws IOException
	 */
	private void streamingInputTestStick(Device device) throws IOException {
		// Construct selection that contains inputs to read from.
		Input[] inputs = new Input[] {
			device.getSingleEndedInputs().get(0),
			device.getSingleEndedInputs().get(1),
			device.getDigitalPorts().get(0)
		};

		// Prepare device with this collection ...
		device.setupInputFrame(inputs);

		// Start sampling ...
		device.startBufferedDataAcquisition(300, 20 * 25, false);

		// Read 20 x 25 frames using blocked read,
		// this function returns after *all* (25) requested frames are collected.
		for (int i = 0; i < 20; ++i) {
			// Read 25 frames ...
			List<InputFrame> frames = device.readBlocking(25);

			// Write out the 1st one.
			InputFrame inFrame = frames.get(0);
			System.out.println(
				"DigitalPort #0: " + inFrame.getDigitalPort(0) + ", " +
				"SingleEnded #0: " + inFrame.getSingleEnded(0) + " V, " +
				"SingleEnded #1: " + inFrame.getSingleEnded(1) + " V");
		}

		// Stop the DAQ.
		device.stopDataAcquisition();
	}

	/**
	 * The examples main method.
	 */
	private void runExample() {
		try {
			// Search for devices ...
			List<Device> devices = LibraryInterface.enumerate(DeviceType.All);

			// If at least one has been found, use the first one ...
			if (!devices.isEmpty()) {
				Device device = devices.get(0);

				// Open device, nothing can be done without doing this.
				device.open();

				// Do the streaming ...
				if (device.getDeviceType() == DeviceType.CeboLC)
					streamingInputTestLC(device);
				else if (device.getDeviceType() == DeviceType.CeboStick)
					streamingInputTestStick(device);
				else
					throw new RuntimeException("No supported device detected.");

				// Finalize device usage, this free's up the device, so it can be used
				// again, including other applications.
				device.close();
			}
		}
		catch (IOException ex) {
			ex.printStackTrace();
		}
	}

	public static void main(String[] args) {
		new StreamingIExample().runExample();
	}
}
